import React, {useState} from "react";
// import logo from './logo.svg';
import './App.css';

import InputComponent from "./components/InputComponent";
import SonComponent from "./components/SonComponent";
import SonComponent2 from "./components/SonComponent2";

function App() {
    // vue3 组合式 API
    // React 必须大写 Pascal 命名 组件
    function FnHello({name = "Fn"}) {
        return <div key={name}>Hello world By {name}</div>
    }

    let com = FnHello({name: "obj"})

    function FnHello2() {
        return React.createElement("div", [], "Hello world By React.createElement")
    }

    // vue2 选项式 API
    class ClassHello extends React.Component {
        // state = {
        //     a: 0
        // }

        constructor(props) {
            super(props);

            this.state = {
                a: 0,
                arr: [0, 1, 2],
                msg: "parent msg"
            }
        }

        subValState = () => {
            this.setState({
                ...this.state,
                a: this.state.a - 1
            })
        }

        addValState() {
            // this.state.a ++
            // this.setState({})
            this.setState((state) => {
                return {
                    ...state,
                    a: ++state.a,
                }
            }, () => {
                console.log("setState callBack, 能获取到修改后的值 => ", this.state.a)
            })

            console.log("setState 之外, 获取不到修改后的值 => ", this.state.a)
        }

        addArr = () => {
            this.setState((state) => {
                state.arr.push(state.arr[state.arr.length - 1] + 1)
                return {
                    ...state,
                    // 解构赋值 使其地址改变 才能触发视图更新
                    // 不直接操作原对象, 要 先拷贝 再操作
                    arr: [...state.arr],
                }
            }, () => {
                console.log("state.arr => ", this.state.arr)
            })
        }

        render() {
            return <div style={{border: '1px solid', width: '80%', overflow: 'auto'}}>
                <div>Hello world By Class</div>
                <div> state => {this.state.a}</div>

                <div style={{display: "flex", justifyContent: "center"}}>
                    <button onClick={this.subValState}>-1</button>
                    <button onClick={this.addValState.bind(this)}>+1</button>
                </div>


                <div>{this.state.arr}</div>
                <div style={{display: "flex", justifyContent: "center"}}>
                    <button onClick={this.addArr}>+1</button>
                </div>
            </div>
        }
    }

    // React 不能直接渲染 js 对象
    let obj = {
        a: 1,
        b: 2
    }

    // 如果是数组则依次遍历
    let arr1 = [
        1,
        2,
        3
    ]

    // 数组渲染
    let arr2 = [
        FnHello({name: "item1"}),
        FnHello({name: "item2"}),
        FnHello({name: "item3"}),
    ]

    function clickHandler2() {
        console.log("按钮2被点击")
    }

    function clickHandler3(arg, e) {
        console.log("按钮3被点击, 参数 " + arg)
        console.log("合成的事件对象", e)
        // 阻止事件冒泡
        e.stopPropagation()
    }

    class IfComponent extends React.PureComponent {
        state = {
            show: true
        }

        show = () => {
            this.setState((state) => ({
                ...state,
                show: true
            }))
        }

        hide = () => {
            this.setState((state) => ({
                ...state,
                show: false
            }))
        }

        render() {
            return <div style={{border: '1px solid', width: '80%', overflow: 'auto'}}>
                <div>条件渲染</div>

                {this.state.show && <div> IfComponent 显示 </div>}

                <div style={{display: 'flex', justifyContent: "center"}}>
                    <button onClick={this.hide}>隐藏</button>
                    <button onClick={this.show}>显示</button>
                </div>
                <button onClick={() => {
                    this.setState((state) => ({
                        ...state,
                        show: !state.show
                    }))
                }}>切换
                </button>
            </div>
        }
    }

    const [msg,setMsg] = useState("parent msg")

    const changeMsg = (newMsg) => {
        setMsg(newMsg)
        return msg;
    }

    return (
        <div className="App">
            {/*class 类名必须为 className, 内容只能是字符串*/}
            <header className="App-header">
                {/*<img src={logo} className="App-logo" alt="logo" />*/}

                {/*数组渲染*/}
                {[
                    // key 必须唯一
                    <FnHello key={1}/>,
                    <FnHello2 key={2}/>
                ]}

                <ClassHello/>

                {com}

                {JSON.stringify(obj)}

                <br/>

                {arr1}

                {arr2}


                {/*style 内联样式 必须写成对象的形式 */}
                <div style={{display: 'flex'}}>
                    {/*事件绑定*/}
                    <button onClick={() => {
                        console.log("按钮1被点击")
                    }}>按钮1
                    </button>

                    <button onClick={clickHandler2}>按钮2</button>
                </div>
                <div>
                    <button onClick={clickHandler3.bind(this, "按钮3")}>按钮3</button>
                    <button onClick={(e) => clickHandler3("按钮3", e)}>按钮3</button>
                </div>

                <IfComponent/>

                <InputComponent/>

                <div style={{
                    border: '1px solid',
                    width: '80%',
                    overflow: 'auto',
                    display: 'flex',
                    flexDirection: 'column',
                    alignItems: 'center',
                    justifyContent: 'center'
                }}>
                    父组件
                    <SonComponent a={1} b={2} msg={"App Msg"}
                                  slot2={<div>SonComponent 插槽 slot2 (具名插槽)</div>}
                                  scopeslot={(scope)=>{return <div>{scope}</div>}}>
                        <div>
                            SonComponent 插槽 slot1
                        </div>
                    </SonComponent>

                    <SonComponent2 msg={msg} changeMsg={changeMsg}></SonComponent2>
                </div>
            </header>
        </div>
    );
}

export default App;
