import React from "react";
import PropTypes from "proptypes"
// 模块化 css, 后缀 .module.css
import SonComponentCss from "./SonComponent.module.css"
import GrandsonComponent from "./GrandSonComponent";

// 类似于 vue 中的 provider + inject
export let Context = React.createContext({})

// let divRef = React.createRef()
export default class SonComponent extends React.Component {
    state = {
        msg: "SonComponent Msg",
        showChildrenBorder: true,
        childrenClass: []
    }

    divRef = React.createRef()

    constructor(props) {
        super(props);
        console.log("[constructor] SonComponent")
    }

    // 相当于 vue mounted
    componentDidMount() {
        this.changeChildrenClass()
        console.log('[挂载] SonComponent 组件后执行的操作');

        // 废弃
        // 旧版本写法
        // console.log(this.refs.devRef)

        // console.log("divRef", divRef, divRef.current)
        console.log("divRef", this.divRef, this.divRef.current)
    }

    // 优化生命周期
    // 手动实现 类似 React.PureComponent 的功能
    // 如果返回值为 false 则不会调用 render 更新组件
    shouldComponentUpdate(nextProps, nextState, nextContext) {
        console.log("[shouldComponentUpdate] SonComponent")
        console.log("修改前 => ", this.state, "修改后 =>", nextState)

        // 是否需要更新组件
        // return !Object.keys(this.state).reduce((prev,key)=> {
        //     return prev?this.state[key] === nextState[key]:prev;
        // }, true)
        for(let key in this.state){
            console.log(key, this.state[key], nextState[key], this.state[key] !== nextState[key])
            if(this.state[key] !== nextState[key]){
                return true;
            }
        }
        return false;
    }

    // 相当于 vue updated
    componentDidUpdate(prevProps, prevState, snapshot) {
        // 当count更新时执行的操作
        console.log('[componentDidUpdate] SonComponent 组件后更新，执行的操作:', this.state);
    }


    // 相当于 vue unmount
    componentWillUnmount() {
        console.log('[卸载] SonComponent 组件 前执行的操作');
    }

    static getDerivedStateFromProps(props, state){
        console.log("[getDerivedStateFromProps] SonComponent", props, state)
        return null
    }

    switchChildrenBorder = () => {
        this.setState({
            ...this.state,
            showChildrenBorder: !this.state.showChildrenBorder
        }, ()=>{
            this.changeChildrenClass()
        })
    }

    changeChildrenClass = () => {
        let childrenClass = []
        if (this.state.showChildrenBorder) {
            childrenClass = [SonComponentCss.children]
        }

        this.setState({
            ...this.state,
            childrenClass
        })
    }

    render() {
        console.log("[render] SonComponent")
        return <div style={{border: '1px solid', width: '80%', overflow: 'auto'}}>
            子组件

            <div ref={this.divRef}>
                {this.state.msg}
            </div>

            <div>
                {this.props.msg}

                <div className={this.state.childrenClass}>
                    {this.props.children}
                </div>

                <div className={this.state.childrenClass}>
                    {this.props.slot2}
                </div>

                <div className={this.state.childrenClass}>
                    {this.props.scopeslot(<div>
                        <div>SonComponent 插槽 slot3 (作用域插槽)</div>
                        <div>
                            {this.state.msg}
                        </div>
                    </div>)}
                </div>

                <div>
                    <button onClick={this.switchChildrenBorder}>切换边框显示</button>
                </div>
            </div>

            {/* 只有 value 一个 props, 如果要传递多个参数则使用对象 */}
            <Context.Provider value={{msg: "从父组件传递的消息"}}>
                <GrandsonComponent/>
            </Context.Provider>
        </div>
    }
}

SonComponent.propTypes = {
    // 自定义类型校验
    // msg: (props)=>{
    //     if(typeof props.msg != "string"){
    //         throw new Error("msg 必须为 string 类型")
    //     }
    // }

    // 使用 proptypes 提供的类型校验
    msg: PropTypes.string
}

SonComponent.defaultProps = {
    msg: "defaultProps"
}
